///[Auth]用户认证
(function(auth) {

	/**
	 * ///[auth:login]用户登陆
	 *
	 * 用户登陆成功后，获取到token及token_expired，并存储到本地。
	 * 
	 * @param {String} loginUrl : 服务端用户登陆URL
	 * @param {Object} loginInfo : 用户登陆信息对象，例如：{"username":"admin","password":"123456"}或{"mobile":"13812345678","password":"123456"}
	 * 		服务端由username或mobile为主键查询对应的用户
	 * @param {function(userInfo)} successCB : 用户登陆成功回调函数，userInfo为用户信息对象
	 * @param {function(err)} errorCB : 用户登陆失败回调函数，err为错误信息
	 **/
	auth.login = function(loginUrl, loginInfo, successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.login"),["loginUrl: "+loginUrl,"loginInfo: "+JSON.stringify(loginInfo)]);

		successCB = successCB || mui.noop;///设置缺省参数
		errorCB = errorCB || mui.noop;///设置缺省参数
		
		///Ajax请求
		console.log("@@@auth.login:Ajax请求 $$$loginUrl:"+loginUrl);
		console.log("@@@auth.login:Ajax请求 $$$loginInfo:"+JSON.stringify(loginInfo));

		mui.ajax(loginUrl, {
			data: loginInfo,
			dataType: 'json',
			type: 'post',
			timeout: 10000,
			success: function(data) {
				console.log("@@@auth.login:Ajax请求成功 $$$data:"+JSON.stringify(data));
				if(data.result=="0"){

					var token_expired = formatTimestamp(data.token.token_expired);///[formatTimestamp]格式化时间戳
					console.log("@@@auth.login:Ajax请求成功 $$$token_expired:"+token_expired);

					///存储到本地
					setSetting("$user", data.userinfo);
					setSetting("$token", data.token);
					
					successCB(data.userinfo);
				}else{
					errorCB(data.reason);
				}
			},
			error: function(xhr, type, errorThrown) {
				//异常处理；
				console.log("Error：Ajax获取返值失败！");
				console.log(xhr.status);
				console.log(xhr.responseText);
				console.log(type);
				console.log(errorThrown);
				errorCB("Error:获取Ajax返值失败!");
			}
		});
	};
	
	/**
	 * ///[auth:register]用户注册
	 *
	 * 用户注册成功后，获取到token及token_expired，并存储到本地。
	 * 
	 * @param {String} regUrl : 服务端用户注册URL
	 * @param {Object} regInfo : 用户注册信息对象，例如：{"username":"admin","password":"123456"}
	 * 或{"mobile":"13812345678","password":"123456"}
	 * 服务端由username或mobile为主键查询对应的用户
	 * @param {function(userInfo)} successCB : 用户注册成功回调函数，userInfo为用户信息对象
	 * @param {function(err)} errorCB ： 用户注册失败回调函数，err为错误信息
	 **/
	auth.reg = function(regUrl, regInfo, successCB, errorCB) {
		auth.login(regUrl,regInfo, 
			function(s) {
				successCB(s);
			},
			function(e) {
				errorCB(e);
			}
		);
	}

	/**
	 * ///[auth:oauthenticate]第三方登陆
	 *
	 * 第三方登陆成功后，到服务端查询对应的用户（如无则创建用户），
	 * 然后获取到token及token_expired，并存储到本地。
	 * 
	 * @param {String} oauthUrl : 服务端第三方登陆URL
	 * @param {Object} oauthInfo : 第三方登陆信息对象，例如：{"id":"qq","description":"QQ","authResult":{"ret":0,"openid":"169BD44A66DD5CDDAA7B50C2983220E3","access_token":"9EA33A98E3DF5E9B7AB29BC3D266CE4B",...}
	 * 服务端根据id获取对应的openid、token、token_expired、头像等信息
	 * @param {function(userInfo)} successCB  第三方登陆成功回调函数，userInfo为用户信息对象
	 * @param {function(err)} errorCB : 第三方登陆失败回调函数，err为错误信息
	 **/
	auth.oauthenticate = function(oauthUrl, oauthInfo, successCB, errorCB) {
		auth.login(oauthUrl,oauthInfo, 
			function(s) {
				successCB(s);
			},
			function(e) {
				errorCB(e);
			}
		);
	};
	
	/**
	 * ///[auth:hasLogin]判断是否为登陆状态
	 * 
	 * @return {boolean} : 如果本地存储有token，则返回true，代表已经登陆；反之则返回false，代表未登录
	 * 
	 * 由于调用了getToken()，所以已经考虑了token过期。
	 **/
	auth.hasLogin = function() {
		var token = auth.getToken();
		return token?true:false;
	}
		
	/**
	 * ///[auth:getUser]获取本地存储的user
	 * 
	 * @return {Object} : 如果无本地存储的user或此token已经过期（未登录），则返回null；否则返回此本地存储的user对象。
	 **/
	auth.getUser = function() {
		var token = auth.getToken();
		if(token){
			var user = getSetting("$user");
			console.log("###获得本地存储的用户对象给内存用户对象 $user:"+JSON.stringify(user));
			return user;
		}else{
			setSetting("$user");
			return null;
		}
	}
	
	/**
	 * ///[auth:getToken]获取本地存储的token
	 * 
	 * @return {String} : 如果无本地存储的token或此token已经过期，则返回null；否则返回此本地存储token。
	 **/
	auth.getToken = function() {
		var token = getSetting("$token");
		if(!token) return null;
		var token_expired = token.token_expired;
		var timestamp = parseInt(new Date().getTime()/1000);    // 当前时间戳（不带毫秒）
		console.log("timestamp:"+timestamp);
		console.log("token_expired:"+token_expired);
		if(token_expired > timestamp){
			return token;
		}else{
			setSetting("$user");
			setSetting("$token");
			return null;
		}
	}

	/**
	 * ///[auth:logout]注销用户，退出登录服务端，清除本地存储token。
	 *
	 * 所有post请求都先校验Token，如果Token无效则不请求。
	 * 
	 * @param {String} logoutUrl : Post提交URL
	 * @param {Integer} uid : 用户uid
	 * @param {function()} successCB : logout成功回调函数
	 * @param {function(err)} errorCB : logout失败回调函数，err为错误信息
	 **/
	auth.logout = function(logoutUrl, uid, successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.logout"),["logoutUrl: "+logoutUrl,"uid: "+uid]);

		successCB = successCB || mui.noop;///设置缺省参数
		errorCB = errorCB || mui.noop;///设置缺省参数

		///校验Token，如果Token无效则不请求
		var token = auth.getToken();
		if(!token) {
			errorCB("Token无效");
			return;
		}
		var post_info = {"uid":uid, "token":token};///拼装post请求信息对象
		
		///Ajax请求
		console.log("@@@auth.logout:Ajax请求 $$$logoutUrl:"+logoutUrl);

		mui.ajax(logoutUrl, {
			data: post_info,
			dataType: 'json',
			type: 'post',
			timeout: 10000,
			success: function(data) {
				console.log("@@@auth.logout:Ajax请求成功 $$$data:"+JSON.stringify(data));
				if(data.result=="0"){
					successCB();
				}else{
					errorCB(data.reason);
				}
			},
			error: function(xhr, type, errorThrown) {
				//异常处理；
				console.log("Error：Ajax获取返值失败！");
				console.log(xhr.status);
				console.log(xhr.responseText);
				console.log(type);
				console.log(errorThrown);
				errorCB("Error:获取Ajax返值失败!");
			}
		});
	};

	
	/**
	 * ///[auth:setAutoLogin]设置自动登陆状态
	 * 
	 * @param {Boolean} state : 为true代表自动登陆，false代表不自动设置
	 **/
	auth.setAutoLogin = function(state) {
		setSetting("$auto_login", state);
	}

	/**
	 * ///[auth:getAutoLogin]获取自动登陆状态
	 * 
	 * @return {Boolean} : 如果设置自动登陆状态则返回true，否则返回false
	 **/
	auth.getAutoLogin = function() {
		return getSetting("$auto_login");;
	}

	/**
	 * ///[auth:oauthenBind]第三方绑定
	 *
	 * 对已经登陆的用户增加openid，同时删除所有其它用户的openid。
	 * 
	 * @param {String} oauthUrl : 服务端第三方绑定URL
	 * @param {Object} oauthInfo : 第三方绑定信息对象，例如：{"id":"qq","description":"QQ","authResult":{"ret":0,"openid":"169BD44A66DD5CDDAA7B50C2983220E3","access_token":"9EA33A98E3DF5E9B7AB29BC3D266CE4B",...}
	 * 服务端根据id获取对应的openid、token、token_expired、头像等信息
	 * @param {Integer} uid : 用户uid
	 * @param {function()} successCB  第三方绑定成功回调函数
	 * @param {function(err)} errorCB : 第三方绑定失败回调函数，err为错误信息
	 **/
	auth.oauthenBind = function(oauthUrl, oauthInfo, uid, successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.oauthenBind"),["oauthUrl: "+oauthUrl,"oauthInfo: "+JSON.stringify(oauthInfo)]);

		successCB = successCB || mui.noop;///设置缺省参数
		errorCB = errorCB || mui.noop;///设置缺省参数

		///校验Token，如果Token无效则不请求
		var token = auth.getToken();
		if(!token) {
			errorCB("Token无效");
			return;
		}
		var auth_info = {"uid":uid, "token":token,"oauthInfo":oauthInfo};///拼装post请求信息对象
		
		///Ajax请求
		console.log("@@@auth.oauthenBind:Ajax请求 $$$oauthUrl:"+oauthUrl);
		console.log("@@@auth.oauthenBind:Ajax请求 $$$auth_info:"+JSON.stringify(auth_info));

		mui.ajax(oauthUrl, {
			data: auth_info,
			dataType: 'json',
			type: 'post',
			timeout: 10000,
			success: function(data) {
				console.log("@@@auth.oauthenBind:Ajax请求成功 $$$data:"+JSON.stringify(data));
				if(data.result=="0"){
					successCB();
				}else{
					errorCB(data.reason);
				}
			},
			error: function(xhr, type, errorThrown) {
				//异常处理；
				console.log("Error：Ajax获取返值失败！");
				console.log(xhr.status);
				console.log(xhr.responseText);
				console.log(type);
				console.log(errorThrown);
				errorCB("Error:获取Ajax返值失败!");
			}
		});
	};
	
	/**
	 * ///[auth:getAvatarByCamera]拍照获取头像
	 *
	 * 启动手机拍照设备进行拍照，获得的图片经过压缩生成头像，并存储在指定的本地目录。
	 * 
	 * @param {String} avatarCapturedPathfile : 本地存储全目录文件名，如"_doc/avatar.jpg"
	 * @param {function(path)} successCB : 拍照获取头像成功回调函数，path为获取的头像本地存储路径，
	 * 		如"/storage/emulated/0/Android/data/io.dcloud.HBuilder/.HBuilder/apps/HBuilder/doc/avatar.jpg"
	 * @param {function(err)} errorCB : 用户登陆失败回调函数，err为错误信息
	 **/
	auth.getAvatarByCamera = function(avatarCapturedPathfile, successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.getAvatarByCamera"),["avatarCapturedPathfile: "+avatarCapturedPathfile]);
		
		errorCB = errorCB || mui.noop;///设置缺省参数
		plus.camera.getCamera().captureImage(function(capturedFile) {
			plus.io.resolveLocalFileSystemURL(capturedFile, function(entry) {
				var path = entry.toLocalURL()+"?version=" + new Date().getTime();///[BUG!!!]避免相同文件名（但内容已经变化后）的缓存问题！
				///console.log(path);
				successCB(path);
			}, function(e) {
				errorCB("读取拍照文件错误：" + e.message);
			});
		}, function(e) {
			errorCB("拍照错误:" + JSON.stringify(e));
		}, {
			filename: avatarCapturedPathfile
		})
	}
	
	/**
	 * ///[auth:getAvatarByGallery]从手机相册中选择获取头像
	 *
	 * 浏览本地存放的图片，获经过压缩生成头像，并存储在指定的本地目录。
	 * 
	 * @param {function(path)} successCB : 获取头像成功回调函数，path为获取的头像本地存储路径，
	 * 		比如"/storage/emulated/0/Android/data/io.dcloud.HBuilder/.HBuilder/apps/HBuilder/doc/avatar_1.jpg?version=1457629006979"
	 * @param {function(err)} errorCB : 用户登陆失败回调函数，err为错误信息
	 **/
	auth.getAvatarByGallery = function(successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.getAvatarByGallery"));

		errorCB = errorCB || mui.noop;///设置缺省参数
		plus.gallery.pick(function(pickfile) {
			plus.io.resolveLocalFileSystemURL(pickfile, function(entry) {
				successCB(pickfile);
			}, function(e) {
				console.log("读取拍照文件错误：" + e.message);
				errorCB("读取拍照文件错误:" + JSON.stringify(e));
			});
		}, function(e) {}, {
			filter: "image"
		})
	};

	/**
	 * ///[auth:compressAvatar]压缩本地头像，覆盖原来头像文件
	 * 
	 * @param {String} src : 本地头像源文件名
	 * @param {String} dst : 本地头像目标文件名
	 * @param {function()} successCB : 上传头像成功回调函数
	 * @param {function()} errorCB : 上传头像失败回调函数
	 **/
	auth.compressAvatar = function(src, dst, callback){
		___debug___ && console.log(___js_func___("Auth.js","auth.compressAvatar"),["src: "+src,"dst: "+dst]);
	
		plus.zip.compressImage({
				src:src,
				dst:dst,
				overwrite:true,
				quality:60,
				width:"128px",
				height:"128px",
			},
			function(event) {
				///console.log("Compress success!");
				callback();
				
			},function(error) {
				///console.log("Compress error!");
				callback();
			}
		);
	}
	
	/**
	 * ///[auth:uploadAvatar]上传头像到服务端
	 *
	 * 浏览本地存放的图片，获经过压缩生成头像，并存储在指定的本地目录。
	 * 
	 * @param {String} avatarUrl : 服务端上传头像URL
	 * @param {String} avatarPathfile : 本地存储全目录文件名，如"_doc/avatar_1.jpg"
	 * @param {Integer} uid : 用户uid
	 * @param {function(upload, status)} completedCB : 上传头像完成回调函数
	 **/
	auth.uploadAvatar = function(avatarUrl, avatarPathfile, uid, completedCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.uploadAvatar"),["avatarUrl: "+avatarUrl,"avatarPathfile: "+avatarPathfile,"uid: "+uid]);

		var task = plus.uploader.createUpload(avatarUrl, 
			{ method:"POST"},
			///{ method:"POST",blocksize:204800,priority:100,retry:2,retryInterval:3,timeout:30 },
			function ( upload, status ) {
				completedCB(upload, status);
			}
		);
		task.addFile( avatarPathfile, {key:"avatar"} );
		task.addData( "uid", uid.toString() );
		task.start();
	}

	/**
	 * ///[auth:downloadAvatar]从服务端下载头像
	 *
	 * 下载头像文件到指定的本地目录，如果本地有头像则覆盖。
	 * 
	 * @param {String} avatarUrl : 服务端下载头像URL
	 * @param {String} avatarPathfile : 本地存储全目录文件名，如"_doc/avatar_1.jpg"
	 * @param {Integer} uid : 用户uid
	 * @param {function(download, status)} completedCB : 下载头像完成回调函数
	 **/
	auth.downloadAvatar = function(avatarUrl, avatarPathfile, uid, completedCB ) {
		___debug___ && console.log(___js_func___("Auth.js","auth.downloadAvatar"),["avatarUrl: "+avatarUrl,"avatarPathfile: "+avatarPathfile,"uid: "+uid]);

		var task = plus.downloader.createDownload(avatarUrl, 
			{ method:"GET",filename:avatarPathfile},
			///{ method:"GET",blocksize:204800,priority:100,retry:2,retryInterval:3,timeout:30 },
			function ( download, status ) {
				// 下载完成
				if ( status == 200 ) { 
					console.log("@@@auth.downloadAvatar:plus.uploader.createDownload ###下载成功 $$$download.url"+download.url);
				} else {
					console.log("@@@auth.downloadAvatar:plus.uploader.createDownload ###下载成功 $$$status"+status);
				}
				completedCB(download, status);
			}
		);
		task.start();
	}
	
	/**
	 * ///[auth:post]提交post请求到服务端
	 *
	 * 所有post请求都先校验Token，如果Token无效则不请求。
	 * 
	 * @param {String} postUrl : Post提交URL
	 * @param {Object} postInfo : Post提交信息对象
	 * @param {Integer} uid : 用户uid
	 * @param {function()} successCB : post成功回调函数
	 * @param {function(err)} errorCB : post失败回调函数，err为错误信息
	 **/
	auth.post = function(postUrl, postInfo, uid, successCB, errorCB) {
		___debug___ && console.log(___js_func___("Auth.js","auth.post"),["postUrl: "+postUrl,"postInfo: "+JSON.stringify(postInfo),"uid: "+uid]);

		successCB = successCB || mui.noop;///设置缺省参数
		errorCB = errorCB || mui.noop;///设置缺省参数
		
		///校验Token，如果Token无效则不请求
		var token = auth.getToken();
		if(!token) {
			errorCB("Token无效");
			return;
		}
		var post_info = {"uid":uid, "token":token, "postInfo":postInfo};///拼装post请求信息对象
		
		///Ajax请求
		console.log("@@@auth.post:Ajax请求 $$$postUrl:"+postUrl);
		console.log("@@@auth.post:Ajax请求 $$$post_info:"+JSON.stringify(post_info));

		mui.ajax(postUrl, {
			data: post_info,
			dataType: 'json',
			type: 'post',
			timeout: 10000,
			success: function(data) {
				console.log("@@@auth.post:Ajax请求成功 $$$data:"+JSON.stringify(data));
				if(data.result=="0"){
					successCB();
				}else{
					setSetting("$user");///清除用户
					setSetting("$token");///清除token
					errorCB(data.reason);
				}
			},
			error: function(xhr, type, errorThrown) {
				//异常处理；
				console.log("Error：Ajax获取返值失败！");
				console.log(xhr.status);
				console.log(xhr.responseText);
				console.log(type);
				console.log(errorThrown);
				errorCB("Error:获取Ajax返值失败!");
			}
		});
	};
	
})(window.Auth = {});